/* style.css - VERY OBVIOUS STYLING */

body {
    background: hotpink !important;
    max-width: 800px;
    margin: 40px auto;
    padding: 20px;
    font-family: 'Comic Sans MS', cursive, sans-serif;
}

h1 {
    color: lime;
    text-align: center;
    font-size: 3em;
    text-shadow: 3px 3px 0px purple;
}

.piece {
    background: yellow;
    border: 5px dashed red;
    padding: 20px;
    margin-bottom: 20px;
    transform: rotate(-1deg);
}

.piece:nth-child(even) {
    transform: rotate(1deg);
    background: cyan;
}